@@ -24,6 +24,7 @@
 #include <config.h>
 #include <math.h>
 #include <stdlib.h>
+#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
 #include <libgnome/gnome-macros.h>
@@ -55,8 +56,8 @@
 
 /* Maximum zoom factor */
 
-#define MAX_ZOOM_FACTOR 20
-#define MIN_ZOOM_FACTOR 0.01
+#define MAX_ZOOM_FACTOR 5
+#define MIN_ZOOM_FACTOR 0.05
 
 /* Private part of the ImageView structure */
 struct _ImageViewPrivate {
@@ -66,6 +67,9 @@ struct _ImageViewPrivate {
 	/* Current zoom factors */
 	double zoomx;
 	double zoomy;
+	
+	/* Minimum Zoom Factor for This Pixbuf */
+	double MIN_ZOOM;
 
 	/* Previous zoom factor and zoom anchor point stored for size_allocate */
 	double old_zoomx;
@@ -531,7 +535,8 @@ apply_brightness_and_contrast_to_pixbuf (ImageView *view,
 	}
 }
 
-apply_transform_to_pixbuf (ImageView *view, GdkPixbuf *pixbuf, int width, int height)
+static void
+apply_transform_to_pixbuf (ImageView *view, GdkPixbuf *pixbuf, int x, int y, int width, int height)
 {
 	char *p;
 	int line;
@@ -590,6 +595,9 @@ paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
 
 	/* Draw background if necessary, in four steps */
 
+
+	
+
 	/* Top */
 	if (yofs > 0) {
 		r.x0 = 0;
@@ -675,12 +683,10 @@ paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
 
 	/* For all other cases, create a temporary pixbuf */
 
-#ifdef PACK_RGBA
-	tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, d.x1 - d.x0, d.y1 - d.y0);
-#else
-	tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, d.x1 - d.x0, d.y1 - d.y0);
-#endif
-
+	tmp = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE , 8, d.x1 - d.x0, d.y1 - d.y0);
+	if (gdk_pixbuf_get_has_alpha (priv->pixbuf))
+		gdk_pixbuf_fill (tmp, 0x00000000);
+	
 	if (!tmp) {
 		g_message ("paint_rectangle(): Could not allocate temporary pixbuf of "
 			   "size (%d, %d); skipping", d.x1 - d.x0, d.y1 - d.y0);
@@ -746,6 +752,7 @@ paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
 
 	/* Draw! */
 
+#if 1
 	gdk_pixbuf_composite_color (priv->pixbuf,
 				    tmp,
 				    0, 0,
@@ -757,19 +764,30 @@ paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
 				    d.x0 - xofs, d.y0 - yofs,
 				    check_size,
 				    check_1, check_2);
+#else
+	gdk_pixbuf_composite (priv->pixbuf,
+			      tmp,
+			      0, 0,
+			      d.x1 - d.x0, d.y1 - d.y0,
+			      -(d.x0 - xofs), -(d.y0 - yofs),
+			      priv->zoomx, priv->zoomy,
+			      unity_zoom (priv) ? GDK_INTERP_NEAREST : interp_type,
+			      255);
+#endif
 
 #ifdef LIBEOG_ETTORE_CHANGES
 	if (apply_brightness_and_contrast)
 		apply_brightness_and_contrast_to_pixbuf (view, tmp, d.x1 - d.x0, d.y1 - d.y0);
 
 	if (priv->transform != NULL)
-		apply_transform_to_pixbuf (view, tmp, d.x1 - d.x0, d.y1 - d.y0);
+		apply_transform_to_pixbuf (view, tmp, d.x0, d.y0, d.x1 - d.x0, d.y1 - d.y0);
 #endif
 
 #ifdef PACK_RGBA
 	pack_pixbuf (tmp);
 #endif
 
+#if 0
 	gdk_draw_rgb_image_dithalign (GTK_WIDGET (view)->window,
 				      GTK_WIDGET (view)->style->black_gc,
 				      d.x0, d.y0,
@@ -778,6 +796,20 @@ paint_rectangle (ImageView *view, ArtIRect *rect, GdkInterpType interp_type)
 				      gdk_pixbuf_get_pixels (tmp),
 				      gdk_pixbuf_get_rowstride (tmp),
 				      d.x0 - xofs, d.y0 - yofs);
+#else
+	if (gdk_pixbuf_get_has_alpha (priv->pixbuf))
+		paint_background (view, &d, rect);
+
+	gdk_pixbuf_render_to_drawable_alpha (tmp,
+					     GTK_WIDGET (view)->window,
+					     0, 0,
+					     d.x0, d.y0,
+					     d.x1 - d.x0, d.y1 - d.y0,
+					     priv->dither,
+					     GDK_PIXBUF_ALPHA_FULL,
+					     0,
+					     d.x0 - xofs, d.y0 - yofs);
+#endif
 
 	g_object_unref (tmp);
 
@@ -1375,14 +1407,15 @@ image_view_scroll_event (GtkWidget *widget, GdkEventScroll *event)
 {
 	ImageView *view;
 	ImageViewPrivate *priv;
-	double zoom_factor;
-	int xofs, yofs;
 
 	view = IMAGE_VIEW (widget);
 	priv = view->priv;
-
+	
 	/* Compute zoom factor and scrolling offsets; we'll only use either of them */
 
+	double zoom_factor;
+	int xofs, yofs;
+
 	xofs = priv->hadj->page_increment / 2; /* same as in gtkscrolledwindow.c */
 	yofs = priv->vadj->page_increment / 2;
 
@@ -1763,15 +1796,17 @@ image_view_set_zoom (ImageView *view, double zoomx, double zoomy,
 	g_return_if_fail (zoomy > 0.0);
 
 	priv = view->priv;
+		
+	image_view_update_min_zoom (view);
 
 	if (zoomx > MAX_ZOOM_FACTOR)
 		zoomx = MAX_ZOOM_FACTOR;
-	else if (zoomx < MIN_ZOOM_FACTOR)
-		zoomx = MIN_ZOOM_FACTOR;
+	else if (zoomx < priv->MIN_ZOOM)
+		zoomx = priv->MIN_ZOOM;
 	if (zoomy > MAX_ZOOM_FACTOR)
 		zoomy = MAX_ZOOM_FACTOR;
-	else if (zoomy < MIN_ZOOM_FACTOR)
-		zoomy = MIN_ZOOM_FACTOR;
+	else if (zoomy < priv->MIN_ZOOM)
+		zoomy = priv->MIN_ZOOM;
 
 	if (DOUBLE_EQUAL (priv->zoomx, zoomx) &&
 	    DOUBLE_EQUAL (priv->zoomy, zoomy))
@@ -2155,6 +2190,25 @@ image_view_class_init (ImageViewClass *class)
 	widget_class->focus_out_event = image_view_focus_out_event;
 }
 
+void
+image_view_update_min_zoom (ImageView *view)
+{
+	ImageViewPrivate *priv = view->priv;
+
+	priv->MIN_ZOOM = MIN_ZOOM_FACTOR;
+	if (priv->pixbuf) {
+		double width = (double) gdk_pixbuf_get_width (priv->pixbuf);
+		double height = (double) gdk_pixbuf_get_height (priv->pixbuf);
+
+		priv->MIN_ZOOM = ((double) GTK_WIDGET (view)->allocation.width)/ width;
+
+		if (((double) GTK_WIDGET (view)->allocation.height) / height < priv->MIN_ZOOM)
+			priv->MIN_ZOOM = ((double) GTK_WIDGET (view)->allocation.height)/ height;
+
+		if (priv->MIN_ZOOM > 1.0)
+			priv->MIN_ZOOM = 1.0;
+	}
+}
 
 #ifdef LIBEOG_ETTORE_CHANGES
 
