@@ -47,7 +47,14 @@ namespace FSpot {
 		public bool Loading
 		{
 			get {
-				return done_reading;
+				return !done_reading;
+			}
+		}
+
+		public bool Prepared
+		{
+			get {
+				return area_prepared;
 			}
 		}
 
@@ -116,8 +123,7 @@ namespace FSpot {
 				; //step
 		}
 
-		private void Close () 
-		{
+		private void Close () {
 			ThumbnailGenerator.Default.PopBlock ();
 				
 			try {
@@ -125,12 +131,9 @@ namespace FSpot {
 				if (loader != null) { 
 					loader.AreaPrepared -= ap;
 					loader.AreaUpdated -= au;
+					// this can throw exceptions
 					loader.Close ();
-					loader.Closed -= ev;
-					loader.Dispose ();
 				}
-
-				loader = null;
 			} catch (System.Exception e) {
 				System.Console.WriteLine (e.ToString ());
 				if (pixbuf != null)
@@ -138,9 +141,16 @@ namespace FSpot {
 
 				pixbuf = null;
 			} finally {
+				if (loader != null) {
+					loader.Closed -= ev;
+					loader.Dispose ();
+				}
+
+				loader = null;
+
 				if (stream != null) 
 					stream.Close ();
-				
+
 				stream = null;
 			}
 		}
@@ -178,8 +188,12 @@ namespace FSpot {
 				}
 
 				if (len <= 0) {
-					if (loader.Pixbuf == null)
+					if (loader.Pixbuf == null) {
+						if (pixbuf != null)
+							pixbuf.Dispose ();
+
 						pixbuf = null;
+					}
 
 					UpdateListeners ();
 					done_reading = true;
