@@ -29,7 +29,7 @@ using ICSharpCode.SharpZipLib.GZip;
 
 namespace FSpot {
 	public class FolderExport : GladeDialog {
-		IPhotoCollection selection;
+		IBrowsableCollection selection;
 		[Glade.Widget] Gtk.ScrolledWindow thumb_scrolledwindow;
 		[Glade.Widget] Gtk.Entry uri_entry;
 		[Glade.Widget] Gtk.Entry name_entry;
@@ -60,7 +60,7 @@ namespace FSpot {
 		FSpot.ThreadProgressDialog progress_dialog;
 		System.Threading.Thread command_thread;
 		
-		public FolderExport (IPhotoCollection selection) : base ("folder_export_dialog")
+		public FolderExport (IBrowsableCollection selection) : base ("folder_export_dialog")
 		{
 			/*
 			Gnome.Vfs.ModuleCallbackFullAuthentication auth = new Gnome.Vfs.ModuleCallbackFullAuthentication ();
@@ -248,7 +248,7 @@ namespace FSpot {
 				size = size_spin.ValueAsInt;
 
 			command_thread = new System.Threading.Thread (new System.Threading.ThreadStart (Upload));
-			command_thread.Name = Mono.Posix.Catalog.GetString ("Transfering Pictures");
+			command_thread.Name = Mono.Posix.Catalog.GetString ("Transferring Pictures");
 
 			//FIXME: get the files/dirs count in a cleaner way than (* 5 + 2(zip) + 9)
 			// selection * 5 (original, mq, lq, thumbs, comments)
@@ -262,7 +262,7 @@ namespace FSpot {
 
 	internal class FolderGallery 
 	{
-		protected IPhotoCollection collection;
+		protected IBrowsableCollection collection;
 		protected string gallery_name;
 		protected string gallery_path;
 		protected bool scale;
@@ -280,12 +280,17 @@ namespace FSpot {
 			public int Width;
 			public int Height;
 			public bool Skip;
-			public ScaleRequest (string name, int width, int height, bool skip)
+			public bool CopyExif;
+
+			public ScaleRequest (string name, int width, int height, bool skip) : this (name, width, height, skip, false) {}
+
+			public ScaleRequest (string name, int width, int height, bool skip, bool exif)
 			{
 				this.Name = name != null ? name : "";
 				this.Width = width;
 				this.Height = height;
 				this.Skip = skip;
+				this.CopyExif = exif;
 			}
 
 			public static ScaleRequest Default = new ScaleRequest ("", 0, 0, false);
@@ -295,7 +300,7 @@ namespace FSpot {
 			}
 		}
 		
-		internal FolderGallery (IPhotoCollection selection, string path, string gallery_name)
+		internal FolderGallery (IBrowsableCollection selection, string path, string gallery_name)
 		{
 			this.collection = selection;
 			this.gallery_name = gallery_name;
@@ -320,8 +325,8 @@ namespace FSpot {
 
 		public void ProcessImage (int image_num)
 		{
-			Photo photo = (Photo) collection [image_num];
-			string photo_path = photo.DefaultVersionPath;
+			IBrowsableItem photo = collection [image_num];
+			string photo_path = photo.DefaultVersionUri.LocalPath;
 			string path;
 			ScaleRequest req;
 
@@ -337,21 +342,32 @@ namespace FSpot {
 			
 			Gdk.Pixbuf img = null;
 			Gdk.Pixbuf scaled = null;
-			
+
+			Exif.ExifData data = new Exif.ExifData (photo_path);
+			if (data != null && data.Handle.Handle == System.IntPtr.Zero)
+				data = null;
+
 			for (int i = 1; i < requests.Length; i++) {
 				req = requests [i];
 				if (scale && req.AvoidScale (size))
 					continue;
 
-				if (img == null)
+				if (img == null) 
 					scaled = PixbufUtils.LoadAtMaxSize (photo_path, req.Width, req.Height);
 				else
 					scaled = PixbufUtils.ScaleToMaxSize (img, req.Width, req.Height, false);
 				
 				MakeDir (SubdirPath (req.Name));
 				path = SubdirPath (req.Name, ImageName (image_num));
-				scaled.Savev (path, "jpeg", pixbuf_keys, pixbuf_values);
+
+
 				
+				if (req.CopyExif && data != null) {
+					PixbufUtils.SaveJpeg (scaled, path, 90, data);
+				} else 
+					scaled.Savev (path, "jpeg", pixbuf_keys, pixbuf_values);
+				
+
 				if (img != null)
 					img.Dispose ();
 			}
@@ -418,12 +434,12 @@ namespace FSpot {
 
 	class OriginalGallery : FolderGallery
 	{
-		public OriginalGallery (IPhotoCollection selection, string path, string name) : base (selection, path, name) 
+		public OriginalGallery (IBrowsableCollection selection, string path, string name) : base (selection, path, name) 
 		{ 
 			requests = new ScaleRequest [] { new ScaleRequest ("hq", 0, 0, false),
 							 new ScaleRequest ("mq", 800, 600, true),
-							 new ScaleRequest ("lq", 640, 480, false),
-							 new ScaleRequest ("thumbs", 120, 90, false) };
+							 new ScaleRequest ("lq", 640, 480, false, true),
+							 new ScaleRequest ("thumbs", 120, 120, false) };
 		}
 
 		public override void Generate ()
@@ -467,8 +483,8 @@ namespace FSpot {
 
 		private void CreateComments(string photo_path, int photo_index)
 		{
-			StreamWriter comment = File.CreateText(SubdirPath  ("comments", AlternateName (photo_index, "txt")));
-			comment.Write("<span>image " + photo_index + "</span> ");
+			StreamWriter comment = File.CreateText(SubdirPath  ("comments", AlternateName (photo_index, ".txt")));
+			comment.Write("<span>photo " + (photo_index + 1) + "</span> ");
 			comment.Write (collection [photo_index].Description + "\n");
 			comment.Close();
 		}
@@ -538,7 +554,7 @@ namespace FSpot {
 		string altstylesheet = "f-spot-simple-white.css";
 		string javascript = "f-spot.js";
 		
-		public HtmlGallery (IPhotoCollection selection, string path, string name) : base (selection, path, name) 
+		public HtmlGallery (IBrowsableCollection selection, string path, string name) : base (selection, path, name) 
 		{ 
 			requests = new ScaleRequest [] { new ScaleRequest ("hq", 0, 0, false),
 							 new ScaleRequest ("mq", 480, 320, false),
@@ -552,7 +568,7 @@ namespace FSpot {
 			
 			base.Generate ();
 			
-			Photo [] photos = (Photo []) collection.Items;
+			IBrowsableItem [] photos = collection.Items;
 			
 			int i;
 			for (i = 0; i < photos.Length; i++)
