@@ -1,4 +1,6 @@
+using System;
 using System.IO;
+using FSpot.Xmp;
 
 namespace FSpot {
 	public interface IThumbnailContainer {
@@ -7,6 +9,7 @@ namespace FSpot {
 
 	public class JpegFile : ImageFile, IThumbnailContainer, SemWeb.StatementSource {
 		private Exif.ExifData exif_data;
+		private XmpFile xmp;
 		
 		public JpegFile (string path) : base (path) 
 		{
@@ -25,6 +28,15 @@ namespace FSpot {
 			}
 		}
 
+		public override Cms.Profile GetProfile ()
+		{
+			using (System.IO.FileStream stream = System.IO.File.OpenRead (path)) {
+				JpegHeader header = new JpegHeader (stream, true);
+				return header.GetProfile ();
+			}
+
+		}
+
 		public override string Description {
 			get {
 				// FIXME this should probably read the raw data because libexif sucks.
@@ -41,14 +53,28 @@ namespace FSpot {
 		public void SetDescription (string value)
 		{
 			string description = value;
-			
+#if USE_UNICODE_COMMENTS		
 			Exif.ExifContent exif_content = this.ExifData.GetContents (Exif.Ifd.Exif);			
 			int len = System.Text.Encoding.BigEndianUnicode.GetByteCount (description);
+			string heading = "UNICODE\0";
+			byte [] data = new byte [len + heading.Length];
+			System.Text.Encoding.ASCII.GetBytes (heading, 0, heading.Length, data, 0);
+			System.Text.Encoding.BigEndianUnicode.GetBytes (description, 0, description.Length, data, heading.Length);
+			exif_content.GetEntry (Exif.Tag.UserComment).SetData (data);
+#else
+			Exif.ExifContent exif_content = this.ExifData.GetContents (Exif.Ifd.Exif);			
+			int len = System.Text.Encoding.ASCII.GetByteCount (description);
 			string heading = "ASCII\0\0\0";
 			byte [] data = new byte [len + heading.Length];
 			System.Text.Encoding.ASCII.GetBytes (heading, 0, heading.Length, data, 0);
 			System.Text.Encoding.ASCII.GetBytes (description, 0, description.Length, data, heading.Length);
 			exif_content.GetEntry (Exif.Tag.UserComment).SetData (data);
+#endif
+		}
+		
+		public void SetXmp (XmpFile xmp)
+		{
+			this.xmp = xmp;
 		}
 
 		private void UpdateMeta ()
@@ -64,8 +90,12 @@ namespace FSpot {
 		{
 			JpegHeader header = new JpegHeader (input);
 			UpdateMeta ();
+			
+			header.SetExif (this.ExifData);
+
+			if (xmp != null)
+				header.SetXmp (xmp);
 
-			header.Exif = this.ExifData;
 			header.Save (output);
 		}
 		
@@ -73,9 +103,6 @@ namespace FSpot {
 		{
 			UpdateMeta ();
 
-			//this.ExifData.Dump ();
-
-#if true //USE_UNSTABLE_JPEG_HEADER_CODE
 			string  temp_path = path;
 			using (System.IO.FileStream stream = System.IO.File.OpenRead (path)) {
 				using (System.IO.Stream output = FSpot.Unix.MakeSafeTemp (ref temp_path)) {
@@ -86,10 +113,6 @@ namespace FSpot {
 				System.IO.File.Delete (temp_path);
 				throw new System.Exception (System.String.Format ("Unable to rename {0} to {1}", temp_path, path));
 			}
-#else 
-			JpegUtils.SaveExif (path, this.ExifData);
-#endif
-
 		}
 		
 		public override void Save (Gdk.Pixbuf pixbuf, System.IO.Stream stream)
@@ -190,10 +213,13 @@ namespace FSpot {
 				System.DateTime time;
 				try {
 					using (Exif.ExifData ed = new Exif.ExifData (path)) {
-						Exif.ExifContent content = ed.GetContents (Exif.Ifd.Exif);
-						Exif.ExifEntry entry = content.GetEntry (Exif.Tag.DateTimeOriginal);
-						time = Exif.ExifUtil.DateTimeFromString (entry.Value); 
-						time = time.ToUniversalTime ();
+						string time_str = "";				
+						time_str = ed.LookupFirstValue (Exif.Tag.DateTimeOriginal);
+
+						if (time_str == null || time_str == "") 
+							time_str = ed.LookupFirstValue (Exif.Tag.DateTime);
+
+						time = Exif.ExifUtil.DateTimeFromString (time_str).ToUniversalTime (); 
 					}
 				} catch (System.Exception e) {
 					time = base.Date;
