@@ -6,7 +6,46 @@ using System.Collections;
 using System.IO;
 using System.Text;
 using System;
+using FSpot;
 
+public class PhotoVersion : FSpot.IBrowsableItem {
+	Photo photo;
+	uint version_id;
+	
+	public System.DateTime Time {
+		get { return photo.Time; }
+	}
+
+	public Tag [] Tags {
+		get { return photo.Tags; }
+	}
+
+	public Uri DefaultVersionUri {
+		get { return UriList.PathToFileUri (photo.GetVersionPath (version_id));}
+	}
+
+	public string Description {
+		get { return photo.Description; }
+	}
+
+	public string Name {
+		get { return photo.GetVersionName (version_id); }
+	}
+
+	public Photo Photo {
+		get { return photo; }
+	}
+
+	public uint VersionId {
+		get { return version_id; }
+	}
+
+	public PhotoVersion (Photo photo, uint version_id)
+	{
+		this.photo = photo;
+		this.version_id = version_id;
+	}
+}
 
 public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	// IComparable 
@@ -35,7 +74,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		if (result == 0)
 			return 0;
 		else 
-			result = CompareImportDate (photo1, photo2);
+			result = CompareDate (photo1, photo2);
 
 		if (result == 0)
 			result = CompareCurrentDir (photo1, photo2);
@@ -49,7 +88,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		return result;
 	}
 
-	private static int CompareImportDate (Photo photo1, Photo photo2)
+	private static int CompareDate (Photo photo1, Photo photo2)
 	{
 		return DateTime.Compare (photo1.time, photo2.time);
 	}
@@ -89,7 +128,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	private string directory_path;
 	private string name;
 
-	public string Path {
+	private string Path {
 		get {
 			return directory_path + "/" + name;
 		}
@@ -158,6 +197,11 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		}
 	}
 
+	public IBrowsableItem GetVersion (uint version_id)
+	{
+		return new PhotoVersion (this, version_id);
+	}
+
 	private uint default_version_id = OriginalVersionId;
 	public uint DefaultVersionId {
 		get {
@@ -210,7 +254,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 			return null;
 	}
 
-	public string GetVersionPath (uint version_id)
+        public string GetVersionPath (uint version_id)
 	{
 		if (version_id == OriginalVersionId)
 			return Path;
@@ -382,6 +426,67 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		return tags.Contains (tag);
 	}
 
+	private static FSpot.Xmp.XmpFile CreateXmp (FSpot.IBrowsableItem item)
+	{
+		FSpot.Xmp.XmpFile xmp = new FSpot.Xmp.XmpFile ();
+
+		Tag [] tags = item.Tags;
+		string [] names = new string [tags.Length];
+		
+		for (int i = 0; i < tags.Length; i++)
+			names [i] = tags [i].Name;
+		
+		FSpot.MetadataStore.Add (xmp, "dc:subject", "rdf:Bag", names);
+		xmp.Dump ();
+
+		return xmp;
+	}
+
+	public void WriteMetadataToImage ()
+	{
+		string path = this.DefaultVersionPath;
+
+		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
+		if (img is FSpot.JpegFile) {
+			FSpot.JpegFile jimg = img as FSpot.JpegFile;
+			
+			jimg.SetDescription (this.Description);
+		
+			FSpot.Xmp.XmpFile xmp = CreateXmp (this);
+			jimg.SetXmp (xmp);
+
+			jimg.SaveMetaData (path);
+		}
+	}
+
+	public uint SaveVersion (Gdk.Pixbuf buffer, bool create_version)
+	{
+		uint version = DefaultVersionId;
+		ImageFile img = ImageFile.Create (DefaultVersionUri.LocalPath);
+		Exif.ExifData exif_data = new Exif.ExifData (DefaultVersionPath);
+
+		// Always create a version if the source is not a jpeg for now.
+		create_version = create_version || !(img is FSpot.JpegFile);
+
+		if (create_version)
+			version = CreateDefaultModifiedVersion (DefaultVersionId, false);
+
+		try {
+			string version_path = GetVersionPath (version);
+			
+			PixbufUtils.SaveJpeg (buffer, version_path, 95, exif_data);
+			FSpot.ThumbnailGenerator.Create (version_path).Dispose ();
+			DefaultVersionId = version;
+			
+		} catch (System.Exception e) {
+			if (create_version)
+				DeleteVersion (version);
+			
+			throw e;
+		}
+		
+		return version;
+	}
 
 	// Constructor
 	public Photo (uint id, uint unix_time, string directory_path, string name)
@@ -401,6 +506,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 
 	public Photo (uint id, uint unix_time, string path)
 		: this (id, unix_time,
+
 			System.IO.Path.GetDirectoryName (path),
 			System.IO.Path.GetFileName (path))
 	{
@@ -409,11 +515,10 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 }
 
 public class PhotoStore : DbStore {
-
 	TagStore tag_store;
+	
 	public static ThumbnailFactory ThumbnailFactory = new ThumbnailFactory (ThumbnailSize.Large);
 
-
 	// FIXME this is a hack.  Since we don't have Gnome.ThumbnailFactory.SaveThumbnail() in
 	// GTK#, and generate them by ourselves directly with Gdk.Pixbuf, we have to make sure here
 	// that the "large" thumbnail directory exists.
@@ -519,47 +624,50 @@ public class PhotoStore : DbStore {
 		command.Connection = Connection;
 
 		command.CommandText =
-			"CREATE TABLE photo_versions (      " +
-			"	photo_id      INTEGER,      " +
-			"       version_id    INTEGER,      " +
-			"       name          STRING        " +
+			"CREATE TABLE photo_versions (    " +
+			"       photo_id        INTEGER,  " +
+			"       version_id      INTEGER,  " +
+			"       name            STRING    " +
 			")";
 
 		command.ExecuteNonQuery ();
 		command.Dispose ();
 	}
 
+
 	public Photo Create (string path, out Pixbuf thumbnail)
 	{
-		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
-		DateTime time = img.Date ();
-		return Create (time, path, out thumbnail);
+		return Create (path, path, out thumbnail);
 	}
 
-	public Photo Create (DateTime time_in_utc, string path, out Pixbuf thumbnail)
+	public Photo Create (string newPath, string origPath, out Pixbuf thumbnail)
 	{
-		uint unix_time = DbUtils.UnixTimeFromDateTime (time_in_utc);
-
+		FSpot.ImageFile img = FSpot.ImageFile.Create (origPath);
+		uint unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
+		string description = img.Description != null ? img.Description : "";
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 
 		command.CommandText = String.Format ("INSERT INTO photos (time, " +
 						     "directory_path, name, description, default_version_id) " +
-						     "       VALUES ({0}, '{1}', '{2}', '', {3})                                       ",
+						     "       VALUES ({0}, '{1}', '{2}', '{3}', {4})",
 						     unix_time,
-						     SqlString (System.IO.Path.GetDirectoryName (path)),
-						     SqlString (System.IO.Path.GetFileName (path)),
+						     SqlString (System.IO.Path.GetDirectoryName (newPath)),
+						     SqlString (System.IO.Path.GetFileName (newPath)),
+						     SqlString (description),
 						     Photo.OriginalVersionId);
 
 		command.ExecuteScalar ();
 		command.Dispose ();
 
 		uint id = (uint) Connection.LastInsertRowId;
-		Photo photo = new Photo (id, unix_time, path);
+		Photo photo = new Photo (id, unix_time, newPath);
 		AddToCache (photo);
 		photo.Loaded = true;
 
-		thumbnail = GenerateThumbnail (path);
+		thumbnail = GenerateThumbnail (newPath);		
+		EmitAdded (photo);
+
 		return photo;
 	}
 
@@ -574,7 +682,6 @@ public class PhotoStore : DbStore {
 		while (reader.Read ()) {
 			uint version_id = Convert.ToUInt32 (reader [0]);
 			string name = reader[1].ToString ();
-
 			photo.AddVersionUnsafely (version_id, name);
 		}
 
@@ -604,7 +711,7 @@ public class PhotoStore : DbStore {
 						     "FROM photo_versions ");
 		
 		SqliteDataReader reader = command.ExecuteReader ();
-
+		
 		while (reader.Read ()) {
 			uint id = Convert.ToUInt32 (reader [0]);
 			Photo photo = LookupInCache (id) as Photo;
@@ -625,6 +732,15 @@ public class PhotoStore : DbStore {
 				
 				photo.AddVersionUnsafely (version_id, name);
 			}
+
+			/*
+			string directory_path = null;
+			if (reader [3] != null)
+				directory_path = reader [3].ToString ();
+			System.Console.WriteLine ("directory_path = {0}", directory_path);
+			*/
+
+
 		}
 	}
 
@@ -700,9 +816,9 @@ public class PhotoStore : DbStore {
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 		command.CommandText = String.Format ("SELECT tag_id, version_id, name " +
-						     "FROM photo_tags, photo_versions " +
-						     "WHERE photo_tags.photo_id = photo_versions.photo_id " +
-						     "AND photo_tags.photo_id = {0}", photo.Id);
+						     "  FROM photo_tags, photo_versions " +
+						     " WHERE photo_tags.photo_id = photo_versions.photo_id " +
+						     "   AND photo_tags.photo_id = {0}", photo.Id);
 
 		SqliteDataReader reader = command.ExecuteReader ();
 
@@ -825,6 +941,8 @@ public class PhotoStore : DbStore {
 
 	public void Remove (Photo []items)
 	{
+		EmitRemoved (items);
+
 		StringBuilder query_builder = new StringBuilder ();
 		StringBuilder tv_query_builder = new StringBuilder ();
 		for (int i = 0; i < items.Length; i++) {
@@ -865,31 +983,7 @@ public class PhotoStore : DbStore {
 
 	public override void Remove (DbItem item)
 	{
-		RemoveFromCache (item);
-
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-
-		command.CommandText = String.Format ("DELETE FROM photos WHERE id = {0}", item.Id);
-		command.ExecuteNonQuery ();
-
-		command.Dispose ();
-
-		command = new SqliteCommand ();
-		command.Connection = Connection;
-
-		command.CommandText = String.Format ("DELETE FROM photo_tags WHERE photo_id = {0}", item.Id);
-		command.ExecuteNonQuery ();
-
-		command.Dispose ();
-
-		command = new SqliteCommand ();
-		command.Connection = Connection;
-
-		command.CommandText = String.Format ("DELETE FROM photo_versions WHERE photo_id = {0}", item.Id);
-		command.ExecuteNonQuery ();
-
-		command.Dispose ();
+		Remove (new Photo [] { (Photo)item });
 	}
 
 	public override void Commit (DbItem item)
@@ -949,6 +1043,8 @@ public class PhotoStore : DbStore {
 			command.ExecuteNonQuery ();
 			command.Dispose ();
 		}
+
+		EmitChanged (item);
 	}
 	
 	public class DateRange 
@@ -986,15 +1082,11 @@ public class PhotoStore : DbStore {
 
 	public Photo [] Query (string query)
 	{
-		//Console.WriteLine ("Query Start {0}", System.DateTime.Now.ToLongTimeString ());
-
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 		command.CommandText = query;
 		SqliteDataReader reader = command.ExecuteReader ();
 		
-		//Console.WriteLine ("Query Mid {0}", System.DateTime.Now);
-
 		ArrayList version_list = new ArrayList ();
 		ArrayList id_list = new ArrayList ();
 		while (reader.Read ()) {
@@ -1016,10 +1108,7 @@ public class PhotoStore : DbStore {
 			id_list.Add (photo);
 		}
 
-		//Console.WriteLine ("Query End {0}", System.DateTime.Now.ToLongTimeString ());
-
 		bool need_load = false;
-		//Console.WriteLine ("Start {0}", System.DateTime.Now);
 		foreach (Photo photo in version_list) {
 			AddToCache (photo);
 			need_load |= !photo.Loaded;
@@ -1035,7 +1124,6 @@ public class PhotoStore : DbStore {
 			//Console.WriteLine ("Skipped Loading Data");
 		}
 
-		//Console.WriteLine ("End {0}", System.DateTime.Now);
 		command.Dispose ();
 
 		return id_list.ToArray (typeof (Photo)) as Photo [];
@@ -1053,7 +1141,18 @@ public class PhotoStore : DbStore {
 						     "     WHERE directory_path = \"{0}\"", dir.FullName);
 
 		return Query (query_string);
-	}	   
+	}
+
+	public Photo [] QueryUntagged ()
+	{
+		StringBuilder query_builder = new StringBuilder ();
+
+		query_builder.Append ("SELECT * FROM photos WHERE id NOT IN " +
+					"(SELECT DISTINCT photo_id FROM photo_tags) " +
+					"ORDER BY time");
+
+		return Query (query_builder.ToString ());
+	}
 
 	public Photo [] Query (Tag [] tags, DateRange range)
 	{
