@@ -50,8 +50,6 @@ public class PhotoView : EventBox {
 
 	FSpot.AsyncPixbufLoader loader = new FSpot.AsyncPixbufLoader ();
 
-	private const double MAX_ZOOM = 5.0;
-
 	public double Zoom {
 		get {
 			return photo_view.Zoom;
@@ -61,6 +59,16 @@ public class PhotoView : EventBox {
 			photo_view.Zoom = value;
 		}
 	}
+	
+	public double NormalizedZoom {
+		get {
+			return photo_view.NormalizedZoom;
+		}
+
+		set {
+			photo_view.NormalizedZoom = value;
+		}
+	}
 
 
 	// Public events.
@@ -197,6 +205,16 @@ public class PhotoView : EventBox {
 			UpdateFinished (this);
 	}
 
+	public void ZoomIn ()
+	{
+		photo_view.ZoomIn ();
+	}
+	
+	public void ZoomOut ()
+	{
+		photo_view.ZoomOut ();
+	}
+
 	// Event handlers.
 	private void HandleButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
