@@ -4,6 +4,16 @@ using System.Runtime.InteropServices;
 using System;
 using System.IO;
 
+/**
+  1        2       3      4         5            6           7          8
+
+888888  888888      88  88      8888888888  88                  88  8888888888
+88          88      88  88      88  88      88  88          88  88      88  88
+8888      8888    8888  8888    88          8888888888  8888888888          88
+88          88      88  88
+88          88  888888  888888
+**/
+
 public enum PixbufOrientation {
 	TopLeft = 1,
 	TopRight = 2,
@@ -16,6 +26,31 @@ public enum PixbufOrientation {
 }
 
 class PixbufUtils {
+		
+	static public PixbufOrientation Rotate270 (PixbufOrientation orientation)
+	{
+		PixbufOrientation [] rot = new PixbufOrientation [] {
+			PixbufOrientation.LeftBottom, 
+			PixbufOrientation.LeftTop,
+			PixbufOrientation.RightTop,
+			PixbufOrientation.RightBottom, 
+			PixbufOrientation.BottomLeft,
+			PixbufOrientation.TopLeft,
+			PixbufOrientation.TopRight,
+			PixbufOrientation.BottomRight
+		};
+
+		orientation = rot [((int)orientation) -1];
+		return orientation;
+	}
+
+	static public PixbufOrientation Rotate90 (PixbufOrientation orientation)
+	{
+		orientation = Rotate270 (orientation);
+		orientation = Rotate270 (orientation);
+		orientation = Rotate270 (orientation);
+		return orientation;
+	}
 
 	public static Pixbuf ErrorPixbuf = PixbufUtils.LoadFromAssembly ("f-spot-question-mark.png");
 	public static Pixbuf LoadingPixbuf = PixbufUtils.LoadFromAssembly ("f-spot-loading.png");
@@ -103,7 +138,7 @@ class PixbufUtils {
 
 		public Pixbuf LoadFromFile (string path)
 		{
-			FileStream fs;
+			FileStream fs = null;
 
 			try {
 				orientation = GetOrientation (path);
@@ -111,7 +146,7 @@ class PixbufUtils {
 				int count;
 				
 				byte [] data = new byte [8192];
-				while (((count = fs.Read (data, 0, 8192)) > 0) && this.Write (data, (uint)count))
+				while (((count = fs.Read (data, 0, 8192)) > 0) && this.Write (data, (ulong)count))
 					;
 				
 				this.Close ();
@@ -123,8 +158,11 @@ class PixbufUtils {
 				}
 				return rotated;
 			} catch (Exception e) {
-				System.Console.WriteLine ("Error loading image {0}", path);
+				System.Console.WriteLine ("Error loading photo {0}", path);
 				return null;
+			} finally {
+				if (fs != null)
+					fs.Close ();
 			}
 
 		}
@@ -150,7 +188,7 @@ class PixbufUtils {
 
 		Gdk.Pixbuf result;
 		if (upscale || (scale < 1.0))
-			result = pixbuf.ScaleSimple (scale_width, scale_height, Gdk.InterpType.Bilinear);
+			result = pixbuf.ScaleSimple (scale_width, scale_height, (scale_width > 20) ? Gdk.InterpType.Bilinear : Gdk.InterpType.Nearest);
 		else
 			result = pixbuf.Copy ();
 
@@ -174,14 +212,14 @@ class PixbufUtils {
 #endif
 	}
 
-	static private Pixbuf LoadFromStream (System.IO.Stream input)
+	static public Pixbuf LoadFromStream (System.IO.Stream input)
 	{
 		Gdk.PixbufLoader loader = new Gdk.PixbufLoader ();
 		byte [] buffer = new byte [8192];
 		int n;
 
 		while ((n = input.Read (buffer, 0, 8192)) != 0)
-			loader.Write (buffer, (uint) n);
+			loader.Write (buffer, (ulong) n);
 		
 		loader.Close ();
 		return loader.Pixbuf;
@@ -212,20 +250,43 @@ class PixbufUtils {
 		}
 	}
 
+	public static void Save (Gdk.Pixbuf pixbuf, System.IO.Stream stream, string type, string [] options, string [] values)
+	{
+		byte [] data;
+
+		data = PixbufUtils.Save (pixbuf, type, options, values);
+		stream.Write (data, 0, data.Length);
+	}
+
 	[DllImport("libgdk_pixbuf-2.0-0.dll")]
 	static extern bool gdk_pixbuf_save_to_bufferv (IntPtr raw, out IntPtr data, out uint length, string type, 
 						       string [] keys, string [] values, out IntPtr error);
+
 					
 	public static byte [] Save (Gdk.Pixbuf pixbuf, string type, string [] options, string [] values)
 	{
 		IntPtr error = IntPtr.Zero;
 		IntPtr data;
 		uint length;
+		string [] terminated_options = null;
+		string [] terminated_values = null;
+
+		if (options != null) {
+			terminated_options = new string [options.Length + 1];
+			Array.Copy (options, terminated_options, options.Length);
+		}
+
+		if (options != null) {
+			terminated_values = new string [values.Length + 1];
+			Array.Copy (values, terminated_values, values.Length);
+		}
+
 		gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
 					    out data, 
 					    out length, 
 					    type,
-					    options, values,
+					    terminated_options,
+					    terminated_values,
 					    out error);
 
 		if (error != IntPtr.Zero) 
@@ -260,6 +321,23 @@ class PixbufUtils {
 		return icon;
 	}
 		
+	static public Pixbuf LoadFromScreen (Gdk.Window win) {
+		Gdk.Screen screen = win.Screen;
+		Drawable d = screen.RootWindow;
+		int monitor = screen.GetMonitorAtWindow (win);
+		Gdk.Rectangle geom = screen.GetMonitorGeometry (monitor);
+		
+		//
+		// We use the screen width and height because that reflects
+		// the current resolution, the RootWindow can actually be different.
+		//
+
+		Pixbuf buf = new Pixbuf (Colorspace.Rgb, false, 8, geom.Width, geom.Height);
+		
+		return buf.GetFromDrawable (d,
+					    d.Colormap, geom.X, geom.Y, 0, 0, 
+					    geom.Width, geom.Height);
+	}
 
 	static public Pixbuf LoadFromScreen () {
 		Screen screen = Display.Default.GetScreen (0);
@@ -449,7 +527,7 @@ class PixbufUtils {
 
 	public static Cms.Format PixbufCmsFormat (Pixbuf buf)
 	{
-		return buf.HasAlpha ? Cms.Format.Rgba8 : Cms.Format.Rgb8;
+		return buf.HasAlpha ? Cms.Format.Rgba8Planar : Cms.Format.Rgb8;
 	}
 
 	public static unsafe void ColorAdjust (Pixbuf src, Pixbuf dest, 
@@ -461,8 +539,11 @@ class PixbufUtils {
 			throw new Exception ("Invalid Dimensions");
 
 		//Cms.Profile eos10d = new Cms.Profile ("/home/lewing/ICCProfiles/EOS-10D-True-Color-Non-Linear.icm");
-		Cms.Profile srgb = Cms.Profile.CreateSRgb ();
-		Cms.Profile bchsw = Cms.Profile.CreateAbstract (10, brightness, contrast,
+		Cms.Profile srgb = Cms.Profile.CreateStandardRgb ();
+
+		Cms.Profile bchsw = Cms.Profile.CreateAbstract (256, 
+								0.0, 
+								brightness, contrast,
 								hue, saturation, src_color, 
 								dest_color);
 
@@ -527,12 +608,8 @@ class PixbufUtils {
 	
 	public static PixbufOrientation GetOrientation (string path)
 	{
-		Exif.ExifData data = new Exif.ExifData (path);
-
-		PixbufOrientation orientation = GetOrientation (data);
-		data.Dispose ();
-
-		return orientation;
+		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
+		return img.Orientation;
 	}
 
 	[DllImport("libgnomeui-2-0.dll")]
@@ -608,13 +685,14 @@ class PixbufUtils {
 			area.Y = total_height - args.Y - args.Height;
 			break;
 		case PixbufOrientation.LeftTop:
-			area.X = area.Y;
-			area.Y = area.X;
+			area.X = args.Y;
+			area.Y = args.X;
 			area.Width = args.Height;
 			area.Height = args.Width;
 			break;
 		case PixbufOrientation.RightBottom:
 			area.X = total_height - args.Y - args.Height;
+			area.Y = total_width - args.X - args.Width;
 			area.Width = args.Height;
 			area.Height = args.Width;
 			break;
@@ -641,15 +719,16 @@ class PixbufUtils {
 	{
 		Gdk.Rectangle area = TransformOrientation (src, args, orientation);
 
-		int step = 512;
+		int step = 256;
 
 		Gdk.Rectangle rect = new Gdk.Rectangle (args.X, args.Y, 
 							Math.Min (step, args.Width),
 							Math.Min (step, args.Height));
 
+		Gdk.Rectangle trect = TransformOrientation (src, rect, orientation);
 		Gdk.Pixbuf tmp = new Gdk.Pixbuf (src.Colorspace, src.HasAlpha, 
 						 src.BitsPerSample,
-						 rect.Height, rect.Width);
+						 trect.Width, trect.Height);
 
 		Gdk.Rectangle subarea;
 		while (rect.Y < args.Y + args.Height) {
@@ -660,14 +739,11 @@ class PixbufUtils {
 			        Gdk.Pixbuf ssub = new Gdk.Pixbuf (src, subarea.X, subarea.Y,
 								  subarea.Width, subarea.Height);
 
-				
 				Gdk.Pixbuf tsub = new Gdk.Pixbuf (tmp, 0, 0, trans.Width, trans.Height);
 
 				CopyWithOrientation (ssub, tsub, orientation);
-				
-				tsub.CopyArea (0, 0, trans.Width, trans.Height, dest, trans.X, trans.Y);
-				
 
+				tsub.CopyArea (0, 0, trans.Width, trans.Height, dest, trans.X, trans.Y);
 				
 				ssub.Dispose ();
 				tsub.Dispose ();
@@ -680,26 +756,8 @@ class PixbufUtils {
 		tmp.Dispose ();
 		return area;
 	}
-
 	// Bindings from libf.
 
-	[DllImport ("libfspot")]
-	static extern IntPtr f_pixbuf_copy_rotate_90 (IntPtr src, bool counter_clockwise);
-
-	public static Pixbuf Rotate90 (Pixbuf src, bool counter_clockwise)
-	{
-		return new Pixbuf (f_pixbuf_copy_rotate_90 (src.Handle, counter_clockwise));
-	}
-
-	[DllImport ("libfspot")]
-	static extern IntPtr f_pixbuf_copy_mirror (IntPtr src, bool mirror, bool flip);
-
-	// FIXME not very readable
-	public static Pixbuf Mirror (Pixbuf src, bool mirror, bool flip)
-	{
-		return new Pixbuf (f_pixbuf_copy_mirror (src.Handle, mirror, flip));
-	}
-
 	[DllImport ("libfspot")]
 	static extern IntPtr f_pixbuf_copy_apply_brightness_and_contrast (IntPtr src, float brightness, float contrast);
 
