@@ -1,3 +1,6 @@
+using Gtk;
+using System;
+
 namespace FSpot {
 	public class SingleView {
 		[Glade.Widget] Gtk.HBox toolbar_hbox;
@@ -12,6 +15,8 @@ namespace FSpot {
 		[Glade.Widget] Gtk.Image near_image;
 		[Glade.Widget] Gtk.Image far_image;
 
+		[Glade.Widget] Gtk.Scale zoom_scale;
+
 		protected Glade.XML xml;
 		private Gtk.Window window;
 		PhotoImageView image_view;
@@ -52,9 +57,12 @@ namespace FSpot {
 			image_view = new PhotoImageView (collection);
 			FSpot.Global.ModifyColors (image_view);
 			FSpot.Global.ModifyColors (image_scrolled);
+			image_view.ZoomChanged += HandleZoomChanged;
 			image_scrolled.Add (image_view);
 			
 			Window.ShowAll ();
+
+			zoom_scale.ValueChanged += HandleZoomScaleValueChanged;
 			
 			ShowToolbar = true;
 			ShowSidebar = collection.Count > 1;
@@ -93,7 +101,7 @@ namespace FSpot {
 		void HandleRotate90Command (object sender, System.EventArgs args) 
 		{
 			RotateCommand command = new RotateCommand (this.Window);
-			if (command.Execute (RotateCommand.Direction.Clockwise, new IBrowsableItem [] { image_view.Item.Current })) {
+			if (command.Execute (RotateDirection.Clockwise, new IBrowsableItem [] { image_view.Item.Current })) {
 				collection.MarkChanged (image_view.Item.Index);
 			}
 		}
@@ -101,15 +109,17 @@ namespace FSpot {
 		void HandleRotate270Command (object sender, System.EventArgs args) 
 		{
 			RotateCommand command = new RotateCommand (this.Window);
-			if (command.Execute (RotateCommand.Direction.Counterclockwise, new IBrowsableItem [] { image_view.Item.Current })) {
+			if (command.Execute (RotateDirection.Counterclockwise, new IBrowsableItem [] { image_view.Item.Current })) {
 				collection.MarkChanged (image_view.Item.Index);
 			}
 		}
 		
 		private void HandleSelectionChanged (FSpot.IBrowsableCollection selection) 
 		{
-			if (selection.Count > 0)
+			if (selection.Count > 0) {
 				image_view.Item.Index = ((IconView.SelectionCollection)selection).Ids[0];
+				zoom_scale.Value = image_view.NormalizedZoom;
+			}
 		}
 
 		private void HandleViewToolbar (object sender, System.EventArgs args)
@@ -165,6 +175,26 @@ namespace FSpot {
 				directory_view.Selection.Add (fsview.View.Item.Index);
 			fsview = null;
 		}
+		
+		public void HandleZoomOut (object sender, System.EventArgs args)
+		{
+			image_view.ZoomOut ();
+		}
+
+		public void HandleZoomIn (object sender, System.EventArgs args)
+		{
+			image_view.ZoomIn ();
+		}
+
+		private void HandleZoomScaleValueChanged (object sender, System.EventArgs args)
+		{
+			image_view.NormalizedZoom = zoom_scale.Value;
+		}
+
+		private void HandleZoomChanged (object sender, System.EventArgs args)
+		{
+			zoom_scale.Value = image_view.NormalizedZoom;
+		}
 
 		private void HandleFileClose (object sender, System.EventArgs args)
 		{
