@@ -64,13 +64,16 @@ public class TagView : Gtk.Widget {
 				icon = category.Icon;
 			}
 			
+			if (icon == null)
+				continue;
+			
 			Pixbuf scaled_icon;
 			if (icon.Width == thumbnail_size) {
 				scaled_icon = icon;
 			} else {
 				scaled_icon = icon.ScaleSimple (thumbnail_size, thumbnail_size, InterpType.Bilinear);
 			}
-			
+
 			scaled_icon.RenderToDrawable (GdkWindow, Style.WhiteGC,
 						      0, 0, tag_x, tag_y, thumbnail_size, thumbnail_size,
 						      RgbDither.None, tag_x, tag_y);
