@@ -48,9 +48,20 @@ class Timer : IDisposable {
 		start = System.DateTime.Now;
 	}
 
+	public System.TimeSpan ElapsedTime {
+		get {
+			return System.DateTime.Now - start;
+		}
+	}
+
+	public void WriteElapsed (string message)
+	{
+		System.Console.WriteLine ("{0} {1} {2}", label, message, ElapsedTime);
+	}
+
 	public void Dispose ()
 	{
-		System.Console.WriteLine ("{0} timer elapsed: {1}", label, (System.DateTime.Now - start));
+		WriteElapsed ("timer stopped:");
 	}
 }
 
@@ -185,7 +196,10 @@ class GtkUtil {
 		} else {
 			i = new Gtk.MenuItem (l);
 		}
-		i.Activated += e;
+
+		if (e != null)
+			i.Activated += e;
+
                 i.Sensitive = enabled;
 		
 		menu.Append (i);
